/*
 *  Copyright 2010 Felix Roethenbacher
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package ch.syabru.nagios.broker.impl;

import java.net.MalformedURLException;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.remote.JMXServiceURL;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.transform.stream.StreamSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.core.io.Resource;
import org.springframework.oxm.Unmarshaller;
import org.springframework.oxm.XmlMappingException;

import ch.syabru.nagios.broker.Configuration;
import ch.syabru.nagios.broker.ConfigurationException;
import ch.syabru.nagios.broker.ExternalCommandFactory;
import ch.syabru.nagios.broker.Matcher;
import ch.syabru.nagios.broker.MatcherException;
import ch.syabru.nagios.broker.MessageHandler;
import ch.syabru.nagios.broker.MessageListener;
import ch.syabru.nagios.broker.MessageListenerManager;
import ch.syabru.nagios.broker.jmx.JmxMatcher;
import ch.syabru.nagios.broker.jmx.JmxMessageListener;
import ch.syabru.nagios.broker.xml.XmlActionProcessServiceCheckResult;
import ch.syabru.nagios.broker.xml.XmlActionScheduleForcedServiceCheck;
import ch.syabru.nagios.broker.xml.XmlActionScheduleServiceCheck;
import ch.syabru.nagios.broker.xml.XmlConfiguration;
import ch.syabru.nagios.broker.xml.XmlJmx;
import ch.syabru.nagios.broker.xml.XmlJmxMBeanServer;
import ch.syabru.nagios.broker.xml.XmlJmxMatch;
import ch.syabru.nagios.broker.xml.XmlJmxNotification;

/**
 * Configuration implementation.
 *
 * @author fr
 *
 */
public class ConfigurationImpl implements Configuration {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    private Unmarshaller unmarshaller;
    private Resource configFile;
    private MessageHandler messageHandler;
    private MessageListenerManager messageListenerManager;
    private List<MessageListener> messageListeners;
    private List<Matcher> matchers;

    private XmlConfiguration config;

    /**
     * Read configuration.
     * @throws ConfigurationException XX
     */
    @PostConstruct
    public void readConfiguration()
    throws ConfigurationException
    {
        logger.info("Reading configuration [{}])",
                configFile.getDescription());
        try {
            StreamSource source = new StreamSource(
                    configFile.getInputStream());
            config = (XmlConfiguration) unmarshaller.unmarshal(source);
        } catch (XmlMappingException e) {
            throw new ConfigurationException(
                    "Error parsing configuration file", e);
        } catch (Exception e) {
            throw new ConfigurationException("Error reading configuration", e);
        }
        if (config.getJmx() != null)
            processJmxConfiguration(config.getJmx());
        else
            logger.warn("No configuration elements found. " +
                    "Please check your configuration file.");
    }

    /**
     * Process JMX configuration.
     * @param xmlJmx JMX XML configuration.
     * @throws ConfigurationException XX
     */
    private void processJmxConfiguration(XmlJmx xmlJmx)
    throws ConfigurationException
    {
        // JMX MBean servers.
        for (XmlJmxMBeanServer server : xmlJmx.getMBeanServer()) {
            JMXServiceURL serviceURL;
            try {
                serviceURL = new JMXServiceURL(server.getJmxServiceUrl());
            } catch (MalformedURLException e) {
                throw new ConfigurationException(
                        "Malformed JMX service URL", e);
            }
            // Notifications.
            for (XmlJmxNotification xmlJmxNotification :
                server.getNotification())
            {
                // Create listener.
                ObjectName objectName;
                try {
                    objectName = new ObjectName(
                            xmlJmxNotification.getObjectName());
                } catch (MalformedObjectNameException e) {
                    throw new ConfigurationException(
                            "Malformed object name", e);
                }
                JmxMessageListener listener = new JmxMessageListener(
                        messageListenerManager, messageHandler, serviceURL,
                        objectName);
                messageListeners.add(listener);
                logger.debug("Created JMX listener [{}, {}]", new Object[] {
                        serviceURL.toString(), objectName.toString()});
                // Create matcher.
                XmlJmxMatch xmlJmxMatch = xmlJmxNotification.getMatch();
                String classNameRegex = xmlJmxMatch.getClassName();
                String messageRegex = xmlJmxMatch.getMessage();
                String typeRegex = xmlJmxMatch.getType();
                String sourceRegex = xmlJmxMatch.getSource();
                // Get external command factory.
                for (Object obj : xmlJmxNotification.getAction().getXmlAction())
                {
                    ExternalCommandFactory factory;
                    if (obj instanceof XmlActionProcessServiceCheckResult) {
                        XmlActionProcessServiceCheckResult action =
                            (XmlActionProcessServiceCheckResult) obj;
                        String hostName = action.getHostName();
                        String serviceDescription =
                            action.getServiceDescription();
                        int returnCode = action.getReturnCode();
                        String pluginOutput = action.getPluginOutput();
                        factory = new ProcessServiceCheckResultCommandFactory(
                                hostName, serviceDescription, returnCode,
                                pluginOutput);
                    } else if (obj instanceof XmlActionScheduleServiceCheck) {
                        XmlActionScheduleServiceCheck action =
                            (XmlActionScheduleServiceCheck) obj;
                        String hostName = action.getHostName();
                        String serviceDescription =
                            action.getServiceDescription();
                        XMLGregorianCalendar xmlCheckTime =
                            action.getCheckTime();
                        Integer delaySeconds =
                            action.getDelaySeconds() != null ?
                                    action.getDelaySeconds() : 0;
                        Date checkTime = null;
                        if (xmlCheckTime != null) {
                            checkTime = xmlCheckTime.toGregorianCalendar().
                                    getTime();
                        }
                        factory = new ScheduleServiceCheckCommandFactory(
                                hostName, serviceDescription, checkTime,
                                delaySeconds);
                    } else if (obj instanceof
                            XmlActionScheduleForcedServiceCheck)
                    {
                        XmlActionScheduleForcedServiceCheck action =
                            (XmlActionScheduleForcedServiceCheck) obj;
                        String hostName = action.getHostName();
                        String serviceDescription =
                            action.getServiceDescription();
                        XMLGregorianCalendar xmlCheckTime =
                            action.getCheckTime();
                        Integer delaySeconds =
                            action.getDelaySeconds() != null ?
                                    action.getDelaySeconds() : 0;
                        Date checkTime = null;
                        if (xmlCheckTime != null) {
                            checkTime = xmlCheckTime.toGregorianCalendar().
                                    getTime();
                        }
                        factory = new ScheduleForcedServiceCheckCommandFactory(
                                hostName, serviceDescription, checkTime,
                                delaySeconds);
                    } else {
                        throw new ConfigurationException("Unknown action [" +
                                obj.getClass().getName() + "]");
                    }
                    try {
                        JmxMatcher matcher = new JmxMatcher(classNameRegex,
                                messageRegex, typeRegex, sourceRegex, factory);
                        matchers.add(matcher);
                    } catch (MatcherException e) {
                        throw new ConfigurationException(
                                "Error creating matcher", e);
                    }
                    logger.debug("Created matcher [{}, {}]", new Object[] {
                            serviceURL.toString(),
                            factory.getClass().getSimpleName()});
                }
            }
        }
    }

    @Required
    public void setUnmarshaller(Unmarshaller unmarshaller) {
        this.unmarshaller = unmarshaller;
    }

    @Required
    public void setConfigFile(Resource configFile) {
        this.configFile = configFile;
    }

    @Required
    public void setMessageListeners(List<MessageListener> messageListeners) {
        this.messageListeners = messageListeners;
    }

    @Required
    public void setMatchers(List<Matcher> matchers) {
        this.matchers = matchers;
    }

    @Required
    public void setMessageHandler(MessageHandler messageHandler) {
        this.messageHandler = messageHandler;
    }

    @Required
    public void setMessageListenerManager(
            MessageListenerManager messageListenerManager)
    {
        this.messageListenerManager = messageListenerManager;
    }
}
