/*
 *  Copyright 2010 Felix Roethenbacher
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package ch.syabru.nagios.broker.impl;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Required;

import ch.syabru.nagios.broker.ExternalCommand;
import ch.syabru.nagios.broker.ExternalCommandHandler;
import ch.syabru.nagios.broker.ExternalCommandHandlerException;
import ch.syabru.nagios.broker.Matcher;
import ch.syabru.nagios.broker.Message;
import ch.syabru.nagios.broker.MessageHandler;

/**
 * Message handler implementation.
 *
 * @author Felix Roethenbacher
 *
 */
public class MessageHandlerImpl implements MessageHandler {
    private final Logger logger =
        LoggerFactory.getLogger(MessageHandlerImpl.class);

    private ExternalCommandHandler commandHandler;
    private List<Matcher> matchers;

    @Override
    public void messageReceived(Message message) {
        logger.debug("Message received [{}]", message);
        for (ExternalCommand command : matches(message)) {
            try {
                commandHandler.submitExternalCommand(command);
            } catch (ExternalCommandHandlerException e) {
                logger.error("Error submitting external command", e);
            }
        }
    }

    /**
     * Get list of matching external commands.
     * @param message Message.
     * @return List of matching external commands.
     */
    private List<ExternalCommand> matches(Message message) {
        ArrayList<ExternalCommand> commands = new ArrayList<ExternalCommand>();
        for (Matcher matcher : matchers) {
            ExternalCommand matchingCommand = matcher.matches(message);
            if (matchingCommand != null)
                commands.add(matchingCommand);
        }
        logger.debug("Matching actions found: {}", commands.size());
        return commands;
    }

    @Required
    public void setMatchers(List<Matcher> matchers) {
        this.matchers = matchers;
    }

    @Required
    public void setCommandHandler(ExternalCommandHandler commandHandler) {
        this.commandHandler = commandHandler;
    }
}
