/*
 *  Copyright 2010 Felix Roethenbacher
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package ch.syabru.nagios.broker.impl;

import java.io.PrintStream;
import java.util.List;

import javax.annotation.PreDestroy;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.support.AbstractApplicationContext;

import ch.syabru.nagios.broker.MessageListener;
import ch.syabru.nagios.broker.MessageListenerException;
import ch.syabru.nagios.broker.NagiosMessageBroker;

/**
 * Nagios message broker main class.
 *
 * @author Felix Roethenbacher
 *
 */
public class NagiosMessageBrokerImpl
implements Runnable, NagiosMessageBroker, ApplicationContextAware
{
    private final Logger logger =
        LoggerFactory.getLogger(NagiosMessageBrokerImpl.class);

    private List<MessageListener> messageListeners;
    private AbstractApplicationContext applicationContext;

    private boolean isDaemon = false;
    private boolean stopping = false;

    @Override
    public void run() {
        if (!isDaemon)
            outputApplicationInfo();
        startMessageListeners();
        while (!stopping) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                logger.error("Thread interrupted", e);
            }
        }
        logger.info("Shutting down ...");
        applicationContext.destroy();
        logger.info("Shutdown completed.");
    }

    /**
     * Start message listeners.
     */
    private void startMessageListeners() {
        for (MessageListener listener : messageListeners) {
            listener.registerListener();
        }
    }

    /**
     * Stop message listeners.
     */
    private void stopMessageListeners() {
        for (MessageListener listener : messageListeners) {
            try {
                listener.unregisterListener();
            } catch (MessageListenerException e) {
                logger.error("Error unregistering message listener", e);
            }
        }
    }

    /**
     * Shut-down method.
     */
    @PreDestroy
    public void destroy() {
        stopMessageListeners();
    }

    @Override
    public void start() {
        Thread t = new Thread(this);
        t.start();
    }

    @Override
    public void stop() {
        stopping = true;
    }

    /**
     * Output application info.
     */
    private void outputApplicationInfo() {
        PrintStream out = System.out;
        out.println("Syabru Nagios Message Broker");
        out.println();
        out.println("Hit 'Enter' to terminate application.");
    }

    @Required
    public void setMessageListeners(List<MessageListener> messageListeners) {
        this.messageListeners = messageListeners;
    }

    public void setIsDaemon(boolean isDaemon) {
        this.isDaemon = isDaemon;
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext)
    {
        this.applicationContext =
            (AbstractApplicationContext) applicationContext;
    }
}
