/*
 *  Copyright 2010 Felix Roethenbacher
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package ch.syabru.nagios.broker.impl;

import java.io.IOException;
import java.io.Writer;

import org.apache.commons.lang.Validate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.syabru.nagios.broker.ExternalCommand;

/**
 * Process service check result external command.
 *
 * @author Felix Roethenbacher
 *
 */
public class ProcessServiceCheckResultCommand implements ExternalCommand {
    private static final Logger logger = LoggerFactory.getLogger(
            ProcessServiceCheckResultCommand.class);

    private static final String COMMAND = "PROCESS_SERVICE_CHECK_RESULT";

    private String hostName;
    private String serviceDescription;
    private int returnCode;
    private String pluginOutput;

    /**
     * C'tor.
     * @param hostName Host name.
     * @param serviceDescription Service description.
     * @param returnCode Return code.
     * @param pluginOutput Plugin output.
     */
    public ProcessServiceCheckResultCommand(String hostName,
            String serviceDescription, int returnCode, String pluginOutput)
    {
        Validate.notNull(hostName, "hostName must not be null");
        Validate.notNull(serviceDescription,
                "serviceDescription must not be null");
        Validate.isTrue(
                returnCode == ExternalCommand.STATE_OK ||
                returnCode == ExternalCommand.STATE_WARNING ||
                returnCode == ExternalCommand.STATE_CRITICAL ||
                returnCode == ExternalCommand.STATE_UNKNOWN);
        Validate.notNull(pluginOutput, "pluginOutput must not be null");
        this.hostName = hostName;
        this.serviceDescription = serviceDescription;
        this.returnCode = returnCode;
        this.pluginOutput = pluginOutput;
    }

    @Override
    public void writeCommand(Writer writer) throws IOException
    {
        StringBuilder sb = new StringBuilder();
        sb.append("[");
        sb.append(Long.toString(System.currentTimeMillis() / 1000));
        sb.append("] ");
        sb.append(COMMAND);
        sb.append(";");
        sb.append(hostName);
        sb.append(";");
        sb.append(serviceDescription);
        sb.append(";");
        sb.append(Integer.toString(returnCode));
        sb.append(";");
        sb.append(pluginOutput);
        sb.append("\n");
        writer.write(sb.toString());
        writer.flush();
        logger.debug(sb.toString());
    }

    public String getHostName() {
        return hostName;
    }

    public String getServiceDescription() {
        return serviceDescription;
    }

    public int getReturnCode() {
        return returnCode;
    }

    public String getPluginOutput() {
        return pluginOutput;
    }
}
